package info.textgrid.middleware;

import java.io.IOException;
import java.io.InputStream;
import java.math.BigInteger;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Map;
import java.util.TimeZone;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

import org.apache.commons.io.IOUtils;
import org.apache.commons.logging.LogFactory;

import info.textgrid.clients.AuthClient;
import info.textgrid.clients.tgauth.AuthClientException;
import info.textgrid.middleware.oaipmh.ListMetadataFormatsType;
import info.textgrid.middleware.oaipmh.MetadataFormatType;
import info.textgrid.middleware.oaipmh.ResumptionTokenType;
import info.textgrid.namespaces.middleware.tgauth.ProjectInfo;

/**
 * <p>
 * OAIPMHUtillities is responsible to implement all necessary functions to implement the arguments
 * of a request.
 * </p>
 * 
 * @author Maximilian Brodhun, SUB Göttingen
 * @version 2019-10-29
 * @since 2014-01-29
 */
public class OAIPMHUtilities {

  // **
  // FINALS
  // **

  public static final String OAIPMH_NAMESPACE = "http://www.openarchives.org/OAI/2.0/";
  public static final String OAIPMH_SCHEMA_LOCATION =
      "http://www.openarchives.org/OAI/2.0/OAI-PMH.xsd";
  public static final String OAIDC_PREFIX = "oai_dc";
  public static final String OAIDC_NAMESPACE = "http://www.openarchives.org/OAI/2.0/oai_dc/";
  public static final String OAIDC_SCHEMA_LOCATION =
      "http://www.openarchives.org/OAI/2.0/oai_dc.xsd";
  public static final String DC_NAMESPACE = "http://purl.org/dc/elements/1.1/";
  public static final String DC_SCHEMA_LOCATION =
      "https://dublincore.org/schemas/xmls/simpledc20021212.xsd";
  public static final String METS_NAMESPACE = "http://www.loc.gov/METS/";
  public static final String METS_SCHEMA_LOCATION = "http://www.loc.gov/standards/mets/mets.xsd";

  // **
  // STATICS
  // **

  private static org.apache.commons.logging.Log log = LogFactory.getLog(OAIPMHImpl.class);

  /**
   * <p>
   * Since TextGrid just supports DC the only returned MetadataFormat is DublinCore.
   * </p>
   * 
   * @return
   */
  public MetadataFormatType setMetadataFormats() {

    MetadataFormatType tgDublinCore = new MetadataFormatType();
    tgDublinCore.setMetadataNamespace("http://www.openarchives.org/OAI/2.0/oai_dc/");
    tgDublinCore.setMetadataPrefix("oai_dc");
    tgDublinCore.setSchema("http://www.openarchives.org/OAI/2.0/oai_dc.xsd");

    return tgDublinCore;
  }

  public static List<ProjectInfo> getProjectList() throws AuthClientException {
	    AuthClient auth = new AuthClient();	    
	    List<ProjectInfo> projectInfos =auth.getAllProjects();
		auth.getAllProjects();
		
		return projectInfos;
  }
  
  public static String getProjectName(String projectID) throws AuthClientException {
	  
	  AuthClient auth = new AuthClient();	  
	  System.out.println(projectID);
	  return auth.getProjectInfo(projectID).getName();
  }
  
  
  public static List<String> urisFromSparqlResponse(InputStream in) {
	  	
	    List<String> urilist = new ArrayList<String>();
		String theString;
		try {
			theString = IOUtils.toString(in, "UTF-8");
			System.out.println(theString);
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

	    try {
	      XMLStreamReader r = XMLInputFactory.newInstance()
	          .createXMLStreamReader(in);
	      while (r.hasNext()) {
	    	  if ((r.next() == XMLStreamConstants.START_ELEMENT)) {
	    			  System.out.println(r.getLocalName());
	    	  }
/*
	        if ((r.next() == XMLStreamConstants.START_ELEMENT) && (r.getLocalName().equals("uri"))) {
	          // if (r.getLocalName().equals("uri")) {
	          String uri = r.getElementText();
	          // log.debug("uri: " + uri);
	          urilist.add(uri);
	          // }
	        }*/
	      }
	    } catch (XMLStreamException e) {
	      log.error("error parsing sparql-result-stream", e);
	    }
	    System.out.println("urilistSize: " + urilist.size());
	    return urilist;
	  }
  
  
  /**
   * <p>
   * Producing the list of all metadata formats.
   * </p>
   * 
   * @return tgRepMetadaFormats
   */
  public ListMetadataFormatsType setMetadataFormatList() {

    ListMetadataFormatsType tgRepMetadataFormats = new ListMetadataFormatsType();
    MetadataFormatType tgDublinCore = setMetadataFormats();
    tgRepMetadataFormats.getMetadataFormat().add(tgDublinCore);

    return tgRepMetadataFormats;
  }

  /**
   * <p>
   * Calculating the current date and give it back as XMLGregorianCalendar.
   * </p>
   * 
   * @return now
   * @throws DatatypeConfigurationException
   */
  public static XMLGregorianCalendar getXMLGregorianCalendarNow()
      throws DatatypeConfigurationException {

    GregorianCalendar gregorianCalendar = new GregorianCalendar();
    DatatypeFactory datatypeFactory = DatatypeFactory.newInstance();
    XMLGregorianCalendar now = datatypeFactory.newXMLGregorianCalendar(gregorianCalendar);

    return now;
  }

  /**
   * <p>
   * Converting a given string representing a date value into the date format required for OAIPMH
   * and give it back as XMLGregorianCalendar.
   * </p>
   * 
   * @param originalDateTimeString
   * @return xmlCal
   * @throws ParseException
   * @throws DatatypeConfigurationException
   */
  public static XMLGregorianCalendar convertDateFormat(String originalDateTimeString)
      throws ParseException, DatatypeConfigurationException {

    SimpleDateFormat tgItemTime = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.S");
    Date date = tgItemTime.parse(originalDateTimeString);
    SimpleDateFormat outFormatter = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
    outFormatter.setTimeZone(TimeZone.getTimeZone("UTC"));
    String output2 = outFormatter.format(date);
    XMLGregorianCalendar xmlCal = DatatypeFactory.newInstance().newXMLGregorianCalendar(output2);

    return xmlCal;
  }

  /**
   * <p>
   * Converting a given string representing a date value into the date format required for OAIPMH
   * and give it back as string.
   * </p>
   * 
   * @param originalDateTimeString
   * @return dateOutputAsString
   * @throws ParseException
   */
  public static String datestampAsString(String originalDateTimeString) throws ParseException {

    SimpleDateFormat tgItemTime = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.S");
    Date date = tgItemTime.parse(originalDateTimeString);
    SimpleDateFormat outFormatter = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
    outFormatter.setTimeZone(TimeZone.getTimeZone("UTC"));
    String dateOutputAsString = outFormatter.format(date);

    return dateOutputAsString;
  }

  /**
   * <p>
   * Returns a resumptionToken for a search, if necessary.
   * </p>
   * 
   * TODO Check if we really must put all these references in!
   * 
   * @param completeListSize
   * @param resumptionToken
   * @param cursorCollector
   * @param scrollID
   * @param searchResponseSize
   * @param i
   * @return A resumption token if applicable, null otherwise.
   */
  public static ResumptionTokenType getResumptionToken(final long completeListSize,
      final String resumptionToken, Map<String, Integer> cursorCollector, final String scrollID,
      final int searchResponseSize, final int i) {

    log.info("Creating a ResumptionToken:\n " +
        "CompleteListSize: " + completeListSize + "\n" +
        "ResumptionToken: " + resumptionToken + "\n" +
        "CursorCollector: " + cursorCollector + "\n" +
        "ScrollID: " + scrollID + "\n" +
        "SearchResponseSize: " + scrollID + "\n" +
        "IterationStep: " + i);

    int cursor;

    // Three cases here:
    // 1. Complete list size is > searchResponseSize and a token already is existing: we need to
    // check hash map!
    // 2. Complete list size is > searchResponseSize and we have no token: we do need one!
    // 3. Complete list size is <= searchResponseSize (we do not need a token! do nothing!)

    if (completeListSize > searchResponseSize) {
      ResumptionTokenType resTokenForResponse = new ResumptionTokenType();

      if (resumptionToken != null && cursorCollector.containsKey(resumptionToken)) {
        cursor = cursorCollector.get(resumptionToken).intValue() + i;
        resTokenForResponse.setCursor(BigInteger.valueOf((long) cursor));
        cursorCollector.put(scrollID, cursor);
      } else {
        resTokenForResponse.setCursor(BigInteger.valueOf(searchResponseSize));
        cursorCollector.put(scrollID, searchResponseSize);
        cursor = 0;
        System.out.println("CURSOR: " + cursor);
      }

      // Set resumption token string if cursor is less then complete list size: More objects can be
      // delivered!
      if (cursor < completeListSize) {
        resTokenForResponse.setValue(scrollID);
      }

      // Remove resumption token string if cursor is bigger then or equal complete list size: This
      // is the last response, no more objects are available!
      if (cursor >= completeListSize) {
        resTokenForResponse.setValue("");
      }

      // Set complete list size.
      resTokenForResponse.setCompleteListSize(BigInteger.valueOf(completeListSize));

      return resTokenForResponse;
    }

    return null;
  }
  
	public static boolean isThisDateValid(String dateToValidate){
		
	    SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.S");
	    
	    dateFormat.setLenient(false);
	    try {
	      dateFormat.parse(dateToValidate.trim());
	    } catch (ParseException pe) {
	      return false;
	    }
	    return true;
	}
  
	public static boolean isThisDateValidToOtherTimeStamp(String dateToValidate){
		
	    SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
	    
	    dateFormat.setLenient(false);
	    dateFormat.setLenient(false);
	    try {
	      dateFormat.parse(dateToValidate.trim());
	    } catch (ParseException pe) {
	      return false;
	    }
	    return true;
	}
	
	
    private static final String[] formats = { 
            "yyyy-MM-dd'T'HH:mm:ss'Z'",   "yyyy-MM-dd'T'HH:mm:ssZ",
            "yyyy-MM-dd'T'HH:mm:ss",      "yyyy-MM-dd'T'HH:mm:ss.SSS'Z'",
            "yyyy-MM-dd'T'HH:mm:ss.SSSZ", "yyyy-MM-dd HH:mm:ss", 
            "MM/dd/yyyy HH:mm:ss",        "MM/dd/yyyy'T'HH:mm:ss.SSS'Z'", 
            "MM/dd/yyyy'T'HH:mm:ss.SSSZ", "MM/dd/yyyy'T'HH:mm:ss.SSS", 
            "MM/dd/yyyy'T'HH:mm:ssZ",     "MM/dd/yyyy'T'HH:mm:ss", 
            "yyyy:MM:dd HH:mm:ss",        "yyyyMMdd", };
    
    
    public static String getFormatOfDate(String d) {
        if (d != null) {
            for (String parse : formats) {
                SimpleDateFormat sdf = new SimpleDateFormat(parse);
                try {
                    sdf.parse(d);
                    return parse;
                } catch (ParseException e) {

                }
            }
        }
		return d;
    }

}
